Exception: UringMachine::Terminate
- Inherits:
-
Exception
- Object
- Exception
- UringMachine::Terminate
- Defined in:
- lib/uringmachine.rb
Overview
Terminate is an exception used to terminate fibers without further bubbling.
Terminate is an exception used to terminate fibers without further bubbling.